#define GLFW_INCLUDE_NONE
#include <GLFW/glfw3.h>
#include "glad.h"

#include "Renderer.h"
#include "Win32AssetManager.h"
#include "TouchInput.h"
#include "Position.h"

#undef APIENTRY
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>

static void errorCallback(int error, const char* desc)
{
    MessageBoxA(NULL, desc, "Error", MB_ICONERROR | MB_OK);
}

int main()
{
    glfwSetErrorCallback(errorCallback);

    if (!glfwInit()) {
        return 1;
    }

    glfwWindowHint(GLFW_CLIENT_API, GLFW_OPENGL_ES_API);
    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 0);
    GLFWwindow* window = glfwCreateWindow(1024, 768, "Krimi Dinner", nullptr, nullptr);
    if (!window) {
        glfwTerminate();
        return 1;
    }
    glfwMakeContextCurrent(window);
    int ret = gladLoadGLES2Loader((GLADloadproc)glfwGetProcAddress);
    if (!ret) {
        MessageBoxA(nullptr, "Failed to load GLES3", "Error", MB_ICONERROR | MB_OK);
        glfwDestroyWindow(window);
        glfwTerminate();
        return 1;
    }

    const char* asset_dir = "app\\src\\main\\assets";
    Win32AssetManager::create(asset_dir);

    Renderer::create();

    // BEISPIELCODE
    Position smiley_pos = {512, 384};
    StringHandle smiley = StringRepository::global->internString("smiley_PNG42.png");
    // ENDE BEISPIELCODE

    glfwSwapInterval(1);
    while (!glfwWindowShouldClose(window)) {
        glfwPollEvents();

        /* Gather keyboard input */

        /* Gather mouse input */
        TouchInputEvent touch_event = {};
        int input_event_count = 0;
        if (glfwGetMouseButton(window, GLFW_MOUSE_BUTTON_LEFT) == GLFW_PRESS) {
            double x, y;
            glfwGetCursorPos(window, &x, &y);
            touch_event.kind = TouchInputEventKind::Tap;
            touch_event.start = {static_cast<float>(x), static_cast<float>(y)};
            touch_event.end = touch_event.start;
            input_event_count = 1;
        }

        int w, h;
        glfwGetFramebufferSize(window, &w, &h);

        // INTEGRATION Rufe hier deine "gameloop"/"update" funktion auf, die als Parameter
        // den GameState, input_events und input_event_count und die Displaygröße bekommen sollte:
        //
        // kUpdate(&m_state, input_events, input_event_count, m_display_width, m_display_height)
        //
        //
        // Die Funktion könnte folgende Definition haben:
        // void kUpdate(GameState* state,
        //              const TouchInputEvent* touch_events,
        //              unsigned int touch_event_count,
        //              float display_width,
        //              float display_height)
        // ENDE INTEGRATION

        // BEISPIELCODE
        static float x = 1.f;
        static float d = -0.01f;
        x += d;
        if (x <= 0.f)
            d *= -1.f;
        else if (x >= 1.f)
            d *= -1.f;

        if (input_event_count > 0) {
            smiley_pos = touch_event.end;
            smiley_pos.x -= 250;
            smiley_pos.y -= 250;
        }

        Renderer::ptr->addRect(100, 100, 500, 500, 0.3f, 0.3f, 0.3f, 1.f);
        Renderer::ptr->addRect(smiley_pos.x, smiley_pos.y, 500, 500, 0.f, x * x, 1.f - x * x, 1.f, smiley);
        // ENDE BEISPIELCODE

        Renderer::ptr->renderFrame(static_cast<float>(w), static_cast<float>(h));

        glfwSwapBuffers(window);
    }

    glfwDestroyWindow(window);
    glfwTerminate();
    return 0;
}